/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.perfmon.linux.internal.launchconfig;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.emf.common.util.URI;
import org.eclipse.hyades.perfmon.PerfmonPlugin;
import org.eclipse.hyades.perfmon.common.internal.CommonModuleTraceLaunchJob;
import org.eclipse.hyades.perfmon.linux.internal.LinuxConstants;
import org.eclipse.hyades.perfmon.linux.internal.LinuxModuleTrace;
import org.eclipse.hyades.perfmon.utils.internal.JobLauncher;
import org.eclipse.hyades.perfmon.utils.internal.LauncherUtils;
import org.eclipse.hyades.perfmon.utils.internal.launch.HostTab;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;


public class LinuxLauncher implements ILaunchConfigurationDelegate {

public LinuxModuleTrace trace;	
	public void launch(ILaunchConfiguration conf, String mode, ILaunch launch, IProgressMonitor monitor) throws CoreException {
		IWorkbench workbench = PlatformUI.getWorkbench();
		workbench.getDisplay().syncExec(new StatConRun(conf,monitor));
	}

	class StatConRun implements Runnable {
		IProgressMonitor monitor;
		ILaunchConfiguration conf;
		public StatConRun(ILaunchConfiguration conf,IProgressMonitor monitor) {
			this.monitor = monitor;
			this.conf = conf;
		}
		public void run() {
			try {

				monitor.setTaskName(PerfmonPlugin.getString("PROGRESS_GETCONFIGS"));

				//get necessary connection info
				IPath smodel_path = new Path(conf.getAttribute("smodel_container","")); 
				PerfmonPlugin.DBG.info("PERFMON SMODEL PATH = "+smodel_path);

				String spath = smodel_path.toString().replaceFirst("platform:/resource/","");

// we may or may not have the platform:/resource thing
				URI smodel_uri = URI.createPlatformResourceURI(spath);
				PerfmonPlugin.DBG.info("PERFMON plat SMODEL URI = "+smodel_uri);
//				if (smodel_path.toString().toLowerCase().startsWith("platform:/resource/")) {
//					smodel_uri = URI.createURI(spath);
//					PerfmonPlugin.DBG.info("PERFMON SMODEL URI = "+smodel_uri);
//				}
								
//				String rac_host = conf.getAttribute("host_rac","");
				String rac_host = conf.getAttribute(HostTab.HOSTNAME+LinuxConstants.AGENT_PREFIX,"");
				String port = ""+conf.getAttribute(HostTab.PORT+LinuxConstants.AGENT_PREFIX,10002);
				
				IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();

				//create the project if necessary
				String project = smodel_uri.toString().replaceFirst("platform:/resource/","");
				PerfmonPlugin.DBG.info("profiling project = "+project);
				
				if (project.indexOf('/') != -1) {
					project = project.substring(0,project.indexOf('/')+1);
				}
				PerfmonPlugin.DBG.info("profiling project = "+project);
				IProject pproject = workspaceRoot.getProject(project);
				if (!pproject.exists()) {
					pproject.create(null);
					PerfmonPlugin.DBG.info("created project OK");
				}
				
				pproject.open(null);
				
				//create any folders under the project
				try {
					String folderpath = smodel_uri.toString().replaceFirst("platform:/resource/","/");
					PerfmonPlugin.DBG.info("profiling project Folder path = "+folderpath);
					IPath path = new Path(folderpath);
					PerfmonPlugin.DBG.info("profiling project IPath = "+path);

					if (path.segmentCount() > 1) {
						PerfmonPlugin.DBG.info("profiling container is subfolder within project - creating if necessary now");
						//user has specified a project and one or more folders in the project as the container
						IFolder folder = ResourcesPlugin.getWorkspace().getRoot().getFolder(path);
						if (!folder.exists()) {
//							pproject.open(null);
							folder.create(true,true,null);
							PerfmonPlugin.DBG.info("created subfolder ok");
						}
					} else {
						PerfmonPlugin.DBG.info("profiling container is top level project");
					}
					
				} catch (Exception e) {
					e.printStackTrace();
				}
					

				String tmonitor = conf.getAttribute("monitor",PerfmonPlugin.getString("DEFAULT_MONITOR"));
				URI monitor_uri = null;
				if (tmonitor.indexOf('/') != -1 || tmonitor.indexOf('\\') != -1) {
//				if (tmonitor.startsWith("platform:/resource/")) {
					//user specified a path
					if (!tmonitor.toLowerCase().endsWith(".trcmxmi")) {
						tmonitor += ".trcmxmi";
					}
					monitor_uri = URI.createPlatformResourceURI(tmonitor);
				} else {
					//user specified a name, not a path
					tmonitor = smodel_uri.toString()+"/"+tmonitor;
					if (!tmonitor.toLowerCase().endsWith(".trcmxmi")) {
						tmonitor += ".trcmxmi";
					}
					monitor_uri = URI.createURI(tmonitor);
				}
				
				PerfmonPlugin.DBG.info("MONITOR PATH = "+monitor_uri);

				trace = new LinuxModuleTrace(null,rac_host,smodel_uri,monitor_uri);
				CommonModuleTraceLaunchJob job = new CommonModuleTraceLaunchJob(PerfmonPlugin.getString("PROGRESS_LAUNCHING"),trace);
				JobLauncher.launch(job);
				
				LauncherUtils.openProfilingView();
				
			} catch (Throwable t) {
				PerfmonPlugin.DBG.logVisibleError(t,PerfmonPlugin.getString("ERROR_LAUNCH_VIEW"),true);
			}
		}
	}

}