/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.statistical.ui.widgets.grapher.internal;

import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.TimeZoomSlider;
import org.eclipse.hyades.statistical.ui.widgets.zoomslider.internal.ZoomSlider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;

public class EnumerationGraph implements TextGraph {

static Color black_color; 
    
double xoffset = 0;
double yoffset = 0;

	TimeZoomSlider xslider;
	ZoomSlider yslider;
	
	Color fg_col;
	int line_width = 1;

	int line_style = SWT.LINE_SOLID;
	
	public void setIndicatorSource(IndicatorSource isource) {
	}

	EnumerationGraphSource source;
	
	/**
	 * Graphs the reference lines for a pair of sliders
	 */
	public EnumerationGraph(TimeZoomSlider xslider, ZoomSlider yslider, Color col, EnumerationGraphSource source) {
		this.xslider = xslider;
		this.yslider = yslider;
		
		this.fg_col = col;
		
		this.source = source;
	}
	
	public BasicGraphSource getGraphSource() {
		return source;	
	}
	
	public double getXMin() {
		return source.getMin();		
	}

	public double getXMax() {
		return source.getMax();
	}


	public double getYMin() {
		return Double.NEGATIVE_INFINITY;		
	}

	public double getYMax() {
		return Double.POSITIVE_INFINITY;		
	}

	public TimeZoomSlider getXSlider() {
		return xslider;	
	}

	public ZoomSlider getYSlider() {
		return yslider;	
	}	
	public void setXSlider(TimeZoomSlider slider) {
		xslider = slider;
	}
	
	public void setYSlider(ZoomSlider slider) {
		yslider = slider;
	}	
	
	public void setForeground(Color col) {
		fg_col = col;
	}
	
	public Color getForeground() {
		return fg_col;
	}
	
	public void setLineWidth(int w) {
		line_width = w;
	}
	
	public int getLineWidth() {
		return line_width;
	}

	public void setLineStyle(int swt_line_style) {
		line_style = swt_line_style;
	}
	
	public int getLineStyle() {
		return line_style;	
	}
	
	public void setUseIndicator(boolean b) {
	}

	public void paintGraph(GC gc, int x, int y, int w, int h) {
		paintGraph(gc,x,y,w,h,0);
	}

	public int paintGraph(GC gc, int x, int y, int w, int h, int stroffset) {

		int yoffset = stroffset;
		int fheight = gc.getFontMetrics().getHeight();

//		int yoffset = index * gc.getFontMetrics().getHeight();
	
		gc.setForeground(fg_col);
		gc.setLineWidth(line_width);	
		gc.setLineStyle(line_style);

		Object prev = null;
		for (int i = x; i < x+w; i++) {
			double t = xslider.pixel2Value(i);
			if ( t >= source.getMin() && t <= source.getMax() ) {
				Object str = source.getValueAt(t);

				if (str != prev && !str.equals(prev)) {

					gc.setForeground(fg_col);

					gc.drawLine(i,y,i,y+h);

					gc.drawString(str.toString(),i+2,yoffset + y+1,true);

					try {
					    gc.setForeground(yslider.getDisplay().getSystemColor(SWT.COLOR_BLACK));
					} catch (Throwable xx) {
						if (black_color == null) { 
						    black_color = new Color(yslider.getDisplay(),0,0,0);
						}
						gc.setForeground(black_color);
					}
					gc.drawString(str.toString(),i+1,yoffset + y,true);

					yoffset += fheight;
					
					if ((yoffset + fheight) > h) {
						yoffset = 0;
					}	
					
				}
				prev = str;
			}
		}

		return yoffset;
	}
	
	public void setStaticScaling(double mult) {
	}
	public double getStaticScaling() {
		return 1.0;
	} 

	public int getPlottingType() {
		return Graph.PLOTTING_TYPE_AVERAGE;
	}
	public void setPlottingType(int type) {
	}
	
	public void setPlottingPeriod(int type, double value) {
	}
	public int getPlottingPeriodType() {
		return Graph.PERIOD_TYPE_TICKS;
	}
	public double getPlottingPeriodValue() {
		return 1.0;
	}
	
	public void setXOffset(double millis) {
		xoffset = millis;
	}
	public double getXOffset() {
		return xoffset;
	}
	
	public void setYOffset(double millis) {
		yoffset = millis;
	}
	public double getYOffset() {
		return yoffset;
	}	
	
	public int getNoDataBehaviour() {
		return Graph.NODATA_DO_NOTHING;
	}
	public void setNoDataBehaviour(int i) {
	}
}