/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ArtifactNewWizard.java,v 1.15 2005/03/11 22:12:44 bjiang Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.wizard;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.hyades.models.common.configuration.CFGArtifact;
import org.eclipse.hyades.models.common.configuration.CFGClass;
import org.eclipse.hyades.models.common.configuration.Common_ConfigurationFactory;
import org.eclipse.hyades.models.common.util.ICommonConstants;
import org.eclipse.hyades.test.ui.TestUI;
import org.eclipse.hyades.test.ui.TestUIExtension;
import org.eclipse.hyades.test.ui.TestUIImages;
import org.eclipse.hyades.test.ui.TestUIPlugin;
import org.eclipse.hyades.test.ui.internal.model.EMFUtil;
import org.eclipse.hyades.test.ui.util.ArtifactUtil;
import org.eclipse.hyades.ui.internal.util.UIUtil;
import org.eclipse.hyades.ui.internal.wizard.HyadesNewWizard;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.ui.PartInitException;

/**
 * Extends the HyadesNewWizard to create the Hyades Generic Artifact.
 * 
 * @author psun
 * @since 0.0.1
 * @author bjiang
 * @since 1.3
 */
public class ArtifactNewWizard extends HyadesNewWizard {
	
	private ArtifactSelectionPage artifactSelectionPage = null;
	private CFGArtifact newArtifact;

	/**
	 * Constructor for LocationNewWizard
	 */
	public ArtifactNewWizard()
	{
		super();
		setWindowTitle(TestUIPlugin.getString("WIZ_ARTIFACT_TTL")); //$NON-NLS-1$
		setDefaultPageImageDescriptor(TestUIImages.INSTANCE.getImageDescriptor(TestUIImages.IMG_WIZBAN_NEWARTIFACT));

		IDialogSettings dialogSettings = TestUIPlugin.getInstance().getDialogSettings();
		IDialogSettings settings = dialogSettings.getSection("AritfactNewWizard"); //$NON-NLS-1$
		if(settings == null)
			settings = dialogSettings.addNewSection("ArtifactNewWizard"); //$NON-NLS-1$
		setDialogSettings(settings);
	}

	/**
	 * Returns the type for the datapool created by this wizard.
	 * @return String
	 */
	protected String getType()
	{
		return TestUIExtension.GENERIC_ARTIFACT_TYPE;
	}
	
	/**
	 * @see org.eclipse.hyades.ui.internal.wizard.HyadesNewWizard#getFileExtension()
	 */
	public String getFileExtension()
	{
		return ICommonConstants.ARTIFACT_FILE_EXTENSION;
	}


	/**
	 * @see org.eclipse.hyades.ui.internal.wizard.HyadesWizard#initPages()
	 */
	protected void initPages()
	{
		super.initPages();
		artifactSelectionPage = new ArtifactSelectionPage("assetSelection"); //$NON-NLS-1$
	}

	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages()
	{
		getLocationPage().setTitle(TestUIPlugin.getString("WIZ_ARTIFACT_GEN_PG_TTL")); //$NON-NLS-1$
		getLocationPage().setDescription(TestUIPlugin.getString("WIZ_ARTIFACT_GEN_PG_LOC_DSC")); //$NON-NLS-1$
		addPage(getLocationPage());
		
		getAttributeWizardPage().setTitle(TestUIPlugin.getString("WIZ_ARTIFACT_GEN_PG_TTL")); //$NON-NLS-1$
		getAttributeWizardPage().setDescription(TestUIPlugin.getString("WIZ_ARTIFACT_GEN_PG_ATT_DSC")); //$NON-NLS-1$
		addPage(getAttributeWizardPage());
		
		artifactSelectionPage.setTitle(TestUIPlugin.getString("WIZ_ARTIFACT_ASS_PG_TTL")); //$NON-NLS-1$
		artifactSelectionPage.setDescription(TestUIPlugin.getString("WIZ_ARTIFACT_ASS_PG_DSC")); //$NON-NLS-1$
		addPage(artifactSelectionPage);
		
	}
	
	/**
	 * @see org.eclipse.hyades.ui.test.internal.wizard.HyadesNewWizard#createObject(org.eclipse.core.resources.IFile)
	 */
	protected boolean createObject(IFile file)
	throws Exception
	{
		Resource resource = createResource(file);
		resource.getContents().add(createEObject());
		EMFUtil.save(resource);
		return true;
	}
	
	/**
	 * Creates the EObject setting the attributes based on the information
	 * the user has defined in this wizard's pages.
	 * @return a not null EObject
	 */
	protected EObject createEObject()
	{
		newArtifact = Common_ConfigurationFactory.eINSTANCE.createCFGArtifact();
		newArtifact.setName(getLocationPage().getItemName());		
		String description = getAttributeWizardPage().getItemDescription();
		if((description != null) && (description.trim().length() > 0))
		newArtifact.setDescription(description);

		EObject[] eObjects = artifactSelectionPage.getSelectedObjects();
		if(eObjects != null && eObjects.length > 0)
		{
			for(int i = 0; i < eObjects.length; i++)
			{
			    if(!(eObjects[i] instanceof CFGClass))
			        continue;
			    
			    ArtifactUtil.associateTestAsset((CFGClass)eObjects[i], newArtifact);
			}
		}
				
		return newArtifact;
	}
	
	/**
	 * Creates the EMF resource in which the object is added to.
	 * @return a not null Resource
	 */
	protected Resource createResource(IFile file)
	{
		URI uri = URI.createPlatformResourceURI(file.getFullPath().toString());
		Resource.Factory factory = EMFUtil.getResourceFactory(ICommonConstants.ARTIFACT_FILE_EXTENSION);
		return factory.createResource(uri);
	}

		
	/**
	 * @see org.eclipse.hyades.ui.internal.wizard.HyadesNewWizard#openEditor(org.eclipse.core.resources.IFile)
	 */
	protected void openEditor(IFile file) 
	throws PartInitException
	{
		if(TestUIPlugin.getInstance().getPreferenceStore().getBoolean(TestUI.OPEN_EDITOR))
			UIUtil.openEditor(file, TestUIExtension.ARTIFACT_EDITOR_PART_ID, false);
	}	
	
	public CFGArtifact getNewArtifact()
	{
		return this.newArtifact;
	}
	
}
